# $Id: importNOS_CurSubs.rb 6953 2018-12-24 21:51:23Z flaterco $
# Functions for importNOS.rb:
# Current subordinate station import

# Extra messiness happens here because the migration of currents to MDAPI is
# in progress and incomplete.  OTOH, the processing is perfunctory because
# most of the refs are missing and the remaining subs are not changing.

def importCurSubs(db)
  currents = readCurJson()
  currents.each {|csta|

    raise "Notices are not null anymore" unless csta["notices"].nil?
    raise "Disclaimers are not null anymore" unless csta["disclaimers"].nil?

    offsets = csta["currentpredictionoffsets"]
    shortrefsid = offsets["refStationId"]

    next if shortrefsid.empty?

    shortsid = csta["id"]
    sidp = shortsid + "_" + csta["currbin"].to_s
    refsidp = shortrefsid + "_" + offsets["refStationBin"].to_s
    cname = csta["name"]

    if !refStationExists(db, refsidp)
      refsidp.upcase!
      next if !refStationExists(db, refsidp)
    end

    # type is H, S, W, or Y.
    #   H = harmonic
    #   S = subordinate
    #   W = weak and variable with null offsets
    #   Y = undocumented ?  Is that like X only 1 more?
    # Current refs have records saying type H, null offsets, and their own
    # ID in the refStationId field.
    type = csta["type"]
    case type
    when "H", "W"
      next
    when "S"
      # proceed
    else
      print name, " is type ", type, "; not handled\n"
      raise "Unknown type not handled"
    end

    # isMaybeNotFreeData - except we have no geogroup.
    # There was a list of 7 hard-coded names in importNOSSC.pgcc, so guess
    # what?
    # Throw out some B.C. subordinate current stations (on Wrangell).
    if ["PCT2576_1", "PCT2586_1", "PCT2591_1", "PCT2601_1", "PCT2606_1",
	"PCT2611_1", "PCT2616_1"].include?(sidp)
      print "Skipped ", sidp, " ", cname, "\n"
      next
    end

    # depthType is B, S, or U.
    #   B = below chart datum
    #   S = below surface
    #   U = unknown
    # depth_ft = csta["depth"]
    # cname += " (Depth " + depth_ft.to_i.to_s + "ft)" unless depth_ft.nil?

    # Match old data by sidplus.
    name, state, country, tz, units = getOldCurSubFields(db, sidp)
    name = name.chomp(" (2)").chomp(" (sub)")

    # * "No new stations"
    # Given 100% matching success rate above, there is no need for
    # construction of names or guessing of states or time zones, and there
    # is no need for gt, sbgt, or currents_preparsed.txt.

    # Also, there are none in the western Aleutians.
    lat = csta["lat"]
    lng = csta["lng"]

    mindir = offsets["meanEbbDir"]    # degrees true
    maxdir = offsets["meanFloodDir"]  # degrees true
    mint   = offsets["mecTimeAdjMin"] # minutes
    maxt   = offsets["mfcTimeAdjMin"] # minutes
    sbft   = offsets["sbfTimeAdjMin"] # minutes
    sbet   = offsets["sbeTimeAdjMin"] # minutes
    minlm  = offsets["mecAmpAdj"]     # multiplier
    maxlm  = offsets["mfcAmpAdj"]     # multiplier

    raise "Nil offsets" if mint.nil? or maxt.nil? or sbft.nil? or
      sbet.nil? or minlm.nil? or maxlm.nil?

    fixednulls = false
    notes = nil
    legal = nil
    minlm, fixednulls = [0.001, true] if minlm <= 0
    maxlm, fixednulls = [0.001, true] if maxlm <= 0
    if fixednulls
      print "Fixed null multipliers at ", sidp, " ", name, "\n"
      notes = "Velocities for this station should be disregarded--use times only.\n";
      legal = "IGNORE VELOCITIES";
    end

    # Time warps and other meridian problems
    # (c.f. almost identical code in TideSubs)
    submerid = TZmerids[tz]
    if submerid.nil?
      print "Missing from TZmerids: ", tz, "\n"
      raise "Missing tz"
    end
    reftz = getRefTimezone(db, refsidp)
    if reftz != tz
      refmerid = TZmerids[reftz]
      if refmerid.nil?
	print "Missing from TZmerids: ", reftz, "\n"
	raise "Missing tz"
      end
      if refmerid != submerid
	print "Time warp at ", sidp, " ", name, "\n"
	print "  Ref ", refsidp, " is ", reftz, " (merid ", refmerid, ")\n"
	print "  Sub is ", tz, " (merid ", submerid, ")\n"
	adjustMin = (refmerid - submerid) * 60
	mint += adjustMin
	maxt += adjustMin
	sbft += adjustMin
	sbet += adjustMin
	notes = append(notes, "Reference is in time zone " + reftz +
		       "; adjusted offsets " + (refmerid-submerid).to_s +
		       " hr\nto undo LST compensation.\n")
      end
    end

    # There shall be no name clashes; there shall be no new renamings.
    raise "Name clash" if nameInUse(db, name)

    xfields = "Credit:NOAA data processed by David Flater for XTide\n https://flaterco.com/xtide/"

    mint = mint.to_s + " minutes"
    maxt = maxt.to_s + " minutes"
    sbft = sbft.to_s + " minutes"
    sbet = sbet.to_s + " minutes"
    comments = nil              # Saved for future use
    res = db.exec("insert into data_sets (name, original_name, station_id_context, station_id, lat, lng, timezone, state, country, units, source, legalese, notes, comments, restriction, xfields, ref_index, min_time_add, min_level_multiply, max_time_add, max_level_multiply, flood_begins, ebb_begins, min_dir, max_dir) values ($1, $2, 'NOS', $3, $4, $5, $6, $7, $8, $9, 'CO-OPS Metadata API', $10, $11, $12, 'Public domain', $13, $14, $15, $16, $17, $18, $19, $20, $21, $22)", [name, csta["name"], sidp, lat, lng, tz, state, country, units, legal, notes, comments, xfields, getRefIndex(db,refsidp), mint, minlm, maxt, maxlm, sbft, sbet, mindir, maxdir])
  }
end
