# $Id: importNOS_Queries.rb 6950 2018-12-24 20:11:03Z flaterco $
# Functions for importNOS.rb:
# Database queries

# You can do thousands of queries or you can do one big query and build a
# hash in Ruby.  No clue which is better, and it's not important.

# Not well documented: each gives hashes while each_row gives arrays.

# Note that since all currents now use sidplus there is no risk of
# accidentally getting the wrong kind of station when looking up by sid.

# Look up name, state, and time zone by station ID in data_sets_old.
def getOldNameStateTz(db, sid)
  query = db.exec("select min(name), min(state), min(timezone) from data_sets_old where station_id = $1", [sid])
  query.values[0] == [nil, nil, nil] ? nil : query.values[0]
end

# Expanded version for CurSubs; exactly one match required.
def getOldCurSubFields(db, sidp)
  query = db.exec("select name, state, country, timezone, units from data_sets_old where datum is null and station_id = $1", [sidp])
  raise "Match failure" if query.ntuples != 1 # "No new stations" *
  query.values[0]
end

# Look up name, state, and time zone by station ID in reference_stations.
def getRefNameStateTz(db, sid)
  query = db.exec("select min(name), min(state), min(timezone) from reference_stations where station_id = $1", [sid])
  query.values[0] == [nil, nil, nil] ? nil : query.values[0]
end

# Build a cheat sheet mapping 3D coordinates to states.
# If errors occur, consider replacing min with avg.
def getOldStateMap(db)
  stateMap = Hash.new
  query = db.exec("select min(lat), min(lng), min(state) from data_sets_old group by station_id")
  query.each_row {|row|
    stateMap[to_xyz(row[0].to_f, row[1].to_f)] = row[2]
  }
  stateMap
end

# Cache the constituent name aliases mapping for NOS.
def getConstituentAliases(db)
  aliases = Hash.new
  query = db.exec("select alias, name from aliases where format = 'NOS'")
  query.each_row {|row| aliases[row[0]] = row[1]}
  aliases
end

# Cache constituent speeds for validation.
def getSpeeds(db)
  speeds = Hash.new
  query = db.exec("select name, speed from constituents")
  query.each_row {|row| speeds[row[0]] = row[1].to_f}
  speeds
end

# Does a station by this name exist?
def nameInUse(db, name)
  query = db.exec("select count(*) from data_sets where name = $1", [name])
  !query.getvalue(0,0).to_i.zero?
end

# Does a reference station with this station ID exist?
def refStationExists(db, sid)
  query = db.exec("select count(*) from reference_stations where station_id = $1", [sid])
  !query.getvalue(0,0).to_i.zero?
end

# Does a reference station with this name exist?
def refStationNamed(db, name)
  query = db.exec("select count(*) from reference_stations where name = $1", [name])
  !query.getvalue(0,0).to_i.zero?
end

# Does any station exist with this station ID in data_sets_old?
def oldStationExists(db, sid)
  query = db.exec("select count(*) from data_sets_old where station_id = $1", [sid])
  !query.getvalue(0,0).to_i.zero?
end

# Get the time zone of a reference station.
def getRefTimezone(db, sid)
  query = db.exec("select timezone from reference_stations where station_id = $1", [sid])
  if query.ntuples != 1
    print "getRefTimezone: unexpected result querying on sid ", sid, "\n"
    raise "ntuples"
  end
  query.getvalue(0,0)
end

# Get the index of a reference station.
def getRefIndex(db, sid)
  query = db.exec("select index from reference_stations where station_id = $1", [sid])
  if query.ntuples != 1
    print "getRefIndex: unexpected result querying on sid ", sid, "\n"
    raise "ntuples"
  end
  query.getvalue(0,0)
end

# Return ids of western Aleutian stations in data_sets_old as an array.
def getWesternAleutians(db)
  query = db.exec("select distinct station_id from data_sets_old where state = 'AK' and lng > 0")
  query.column_values(0)
end
